@@ -1042,6 +1042,8 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		} else if (strcmp(buf, "test_socket") == 0) {
 			free(conf->test_socket);
 			conf->test_socket = strdup(pos);
+		} else if (strcmp(buf, "use_pae_group_addr") == 0) {
+			conf->use_pae_group_addr = atoi(pos);
 		} else {
 			printf("Line %d: unknown configuration item '%s'\n",
 			       line, buf);
