@@ -49,6 +49,7 @@ struct driver_ops {
 	int (*sta_remove)(void *priv, u8 *addr);
 	int (*get_ssid)(void *priv, u8 *buf, int len);
 	int (*set_ssid)(void *priv, u8 *buf, int len);
+	int (*set_countermeasures)(void *priv, int enabled);
 	int (*send_mgmt_frame)(void *priv, const void *msg, size_t len,
 			       int flags);
 	int (*set_assoc_ap)(void *priv, u8 *addr);
@@ -228,6 +229,14 @@ hostapd_set_assoc_ap(struct hostapd_data *hapd, u8 *addr)
 	return hapd->driver->set_assoc_ap(hapd->driver, addr);
 }
 
+static inline int 
+hostapd_set_countermeasures(struct hostapd_data *hapd, int enabled)
+{
+	if (hapd->driver == NULL || hapd->driver->set_countermeasures == NULL)
+		return 0;
+	return hapd->driver->set_countermeasures(hapd->driver, enabled);
+}
+
 static inline int
 hostapd_sta_add(struct hostapd_data *hapd, u8 *addr, u16 aid, u16 capability,
 		u8 tx_supp_rates)
