@@ -375,6 +375,19 @@ SM_STATE(BE_AUTH, REQUEST)
 	txReq();
 	sm->be_auth.eapReq = FALSE;
 	sm->be_auth.backendOtherRequestsToSupplicant++;
+
+	/*
+	 * Clearing eapolEap here is not specified in IEEE Std 802.1X-2004, but
+	 * it looks like this would be logical thing to do there since the old
+	 * EAP response would not be valid anymore after the new EAP request
+	 * was sent out.
+	 *
+	 * A race condition has been reported, in which hostapd ended up
+	 * sending out EAP-Response/Identity as a response to the first
+	 * EAP-Request from the main EAP method. This can be avoided by
+	 * clearing eapolEap here.
+	 */
+	sm->eapolEap = FALSE;
 }
 
 
@@ -768,9 +781,17 @@ static void eapol_sm_step_run(struct eapol_state_machine *sm)
 	u8 addr[ETH_ALEN];
 	int prev_auth_pae, prev_be_auth, prev_reauth_timer, prev_auth_key_tx,
 		prev_key_rx, prev_ctrl_dir;
+	int max_steps = 100;
 
 	memcpy(addr, sm->sta->addr, ETH_ALEN);
 
+	/*
+	 * Allow EAPOL state machines to run as long as there are state
+	 * changes, but exit and return here through event loop if more than
+	 * 100 steps is needed as a precaution against infinite loops inside
+	 * eloop callback.
+	 */
+restart:
 	prev_auth_pae = sm->auth_pae.state;
 	prev_be_auth = sm->be_auth.state;
 	prev_reauth_timer = sm->reauth_timer.state;
@@ -796,6 +817,8 @@ static void eapol_sm_step_run(struct eapol_state_machine *sm)
 	    prev_auth_key_tx != sm->auth_key_tx.state ||
 	    prev_key_rx != sm->key_rx.state ||
 	    prev_ctrl_dir != sm->ctrl_dir.state) {
+		if (--max_steps > 0)
+			goto restart;
 		/* Re-run from eloop timeout */
 		eapol_sm_step(sm);
 		return;
@@ -803,6 +826,8 @@ static void eapol_sm_step_run(struct eapol_state_machine *sm)
 
 	if (eapol_sm_sta_entry_alive(hapd, addr) && sm->eap) {
 		if (eap_sm_step(sm->eap)) {
+			if (--max_steps > 0)
+				goto restart;
 			/* Re-run from eloop timeout */
 			eapol_sm_step(sm);
 			return;
