@@ -44,6 +44,7 @@
 #include "tls.h"
 #include "eap_sim_db.h"
 #include "version.h"
+#include "hostap_common.h"
 
 
 struct hapd_interfaces {
@@ -201,6 +202,12 @@ static void hostapd_deauth_all_stas(hostapd *hapd)
 /* This function will be called whenever a station associates with the AP */
 void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta, int reassoc)
 {
+	if (hapd->tkip_countermeasures) {
+		hostapd_sta_deauth(hapd, sta->addr,
+				   WLAN_REASON_MICHAEL_MIC_FAILURE);
+		return;
+	}
+
 	/* IEEE 802.11F (IAPP) */
 	if (hapd->conf->ieee802_11f)
 		iapp_new_station(hapd->iapp, sta);
