@@ -368,8 +368,7 @@ hostapd_acl_recv_radius(struct radius_msg *msg, struct radius_msg *req,
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "Found matching Access-Request "
 		      "for RADIUS message (id=%d)\n", query->radius_id);
 
-	if (radius_msg_verify_acct(msg, shared_secret, shared_secret_len,
-				   req)) {
+	if (radius_msg_verify(msg, shared_secret, shared_secret_len, req, 0)) {
 		printf("Incoming RADIUS packet did not have correct "
 		       "authenticator - dropped\n");
 		return RADIUS_RX_INVALID_AUTHENTICATOR;
