@@ -34,6 +34,8 @@
 #include "eloop.h"
 #include "sta_info.h"
 #include "l2_packet.h"
+#include "accounting.h"
+#include "hostap_common.h"
 
 
 static void wpa_send_eapol_timeout(void *eloop_ctx, void *timeout_ctx);
@@ -396,8 +398,8 @@ static void wpa_rekey_gtk(void *eloop_ctx, void *timeout_ctx)
 
 #ifdef CONFIG_RSN_PREAUTH
 
-static void rsn_preauth_receive(void *ctx, unsigned char *src_addr,
-				unsigned char *buf, size_t len)
+static void rsn_preauth_receive(void *ctx, const u8 *src_addr,
+				const u8 *buf, size_t len)
 {
 	struct rsn_preauth_interface *piface = ctx;
 	struct hostapd_data *hapd = piface->hapd;
@@ -480,13 +482,12 @@ static int rsn_preauth_iface_add(struct hostapd_data *hapd, const char *ifname)
 	}
 
 	piface->l2 = l2_packet_init(piface->ifname, NULL, ETH_P_PREAUTH,
-				    rsn_preauth_receive, piface);
+				    rsn_preauth_receive, piface, 1);
 	if (piface->l2 == NULL) {
 		printf("Failed to open register layer 2 access to "
 		       "ETH_P_PREAUTH\n");
 		goto fail2;
 	}
-	l2_packet_set_rx_l2_hdr(piface->l2, 1);
 
 	piface->next = hapd->preauth_iface;
 	hapd->preauth_iface = piface;
@@ -551,6 +552,16 @@ static int rsn_preauth_iface_init(struct hostapd_data *hapd)
 }
 
 
+static void rsn_preauth_finished_cb(void *eloop_ctx, void *timeout_ctx)
+{
+	struct hostapd_data *hapd = eloop_ctx;
+	struct sta_info *sta = timeout_ctx;
+	wpa_printf(MSG_DEBUG, "RSN: Removing pre-authentication STA entry for "
+		   MACSTR, MAC2STR(sta->addr));
+	ap_free_sta(hapd, sta);
+}
+
+
 void rsn_preauth_finished(struct hostapd_data *hapd, struct sta_info *sta,
 			  int success)
 {
@@ -565,7 +576,11 @@ void rsn_preauth_finished(struct hostapd_data *hapd, struct sta_info *sta,
 		pmksa_cache_add(hapd, sta, key, dot11RSNAConfigPMKLifetime);
 	}
 
-	ap_free_sta(hapd, sta);
+	/*
+	 * Finish STA entry removal from timeout in order to avoid freeing
+	 * STA data before the caller has finished processing.
+	 */
+	eloop_register_timeout(0, 0, rsn_preauth_finished_cb, hapd, sta);
 }
 
 
@@ -598,8 +613,8 @@ void rsn_preauth_send(struct hostapd_data *hapd, struct sta_info *sta,
 	ethhdr->h_proto = htons(ETH_P_PREAUTH);
 	memcpy(ethhdr + 1, buf, len);
 
-	if (l2_packet_send(piface->l2, (u8 *) ethhdr, sizeof(*ethhdr) + len) <
-	    0) {
+	if (l2_packet_send(piface->l2, sta->addr, ETH_P_PREAUTH, (u8 *) ethhdr,
+			   sizeof(*ethhdr) + len) < 0) {
 		printf("Failed to send preauth packet using l2_packet_send\n");
 	}
 	free(ethhdr);
@@ -616,6 +631,10 @@ static inline void rsn_preauth_iface_deinit(struct hostapd_data *hapd)
 {
 }
 
+static void rsn_preauth_finished_cb(void *eloop_ctx, void *timeout_ctx)
+{
+}
+
 void rsn_preauth_finished(struct hostapd_data *hapd, struct sta_info *sta,
 			  int success)
 {
@@ -807,6 +826,16 @@ static void rsn_pmkid(const u8 *pmk, const u8 *aa, const u8 *spa, u8 *pmkid)
 static void pmksa_cache_set_expiration(struct hostapd_data *hapd);
 
 
+static void _pmksa_cache_free_entry(struct rsn_pmksa_cache *entry)
+{
+	if (entry == NULL)
+		return;
+	free(entry->identity);
+	ieee802_1x_free_radius_class(&entry->radius_class);
+	free(entry);
+}
+
+
 static void pmksa_cache_free_entry(struct hostapd_data *hapd,
 				   struct rsn_pmksa_cache *entry)
 {
@@ -846,7 +875,7 @@ static void pmksa_cache_free_entry(struct hostapd_data *hapd,
 		prev = pos;
 		pos = pos->next;
 	}
-	free(entry);
+	_pmksa_cache_free_entry(entry);
 }
 
 
@@ -882,6 +911,54 @@ static void pmksa_cache_set_expiration(struct hostapd_data *hapd)
 }
 
 
+static void pmksa_cache_from_eapol_data(struct rsn_pmksa_cache *entry,
+					struct eapol_state_machine *eapol)
+{
+	if (eapol == NULL)
+		return;
+
+	if (eapol->identity) {
+		entry->identity = malloc(eapol->identity_len);
+		if (entry->identity) {
+			entry->identity_len = eapol->identity_len;
+			memcpy(entry->identity, eapol->identity,
+			       eapol->identity_len);
+		}
+	}
+
+	ieee802_1x_copy_radius_class(&entry->radius_class,
+				     &eapol->radius_class);
+}
+
+
+static void pmksa_cache_to_eapol_data(struct rsn_pmksa_cache *entry,
+				      struct eapol_state_machine *eapol)
+{
+	if (entry == NULL || eapol == NULL)
+		return;
+
+	if (entry->identity) {
+		free(eapol->identity);
+		eapol->identity = malloc(entry->identity_len);
+		if (eapol->identity) {
+			eapol->identity_len = entry->identity_len;
+			memcpy(eapol->identity, entry->identity,
+			       entry->identity_len);
+		}
+		wpa_hexdump_ascii(MSG_DEBUG, "STA identity from PMKSA",
+				  eapol->identity, eapol->identity_len);
+	}
+
+	ieee802_1x_free_radius_class(&eapol->radius_class);
+	ieee802_1x_copy_radius_class(&eapol->radius_class,
+				     &entry->radius_class);
+	if (eapol->radius_class.attr) {
+		wpa_printf(MSG_DEBUG, "Copied %lu Class attribute(s) from "
+			   "PMKSA", (unsigned long) eapol->radius_class.count);
+	}
+}
+
+
 void pmksa_cache_add(struct hostapd_data *hapd, struct sta_info *sta, u8 *pmk,
 		     int session_timeout)
 {
@@ -903,6 +980,7 @@ void pmksa_cache_add(struct hostapd_data *hapd, struct sta_info *sta, u8 *pmk,
 		entry->expiration += dot11RSNAConfigPMKLifetime;
 	entry->akmp = WPA_KEY_MGMT_IEEE8021X;
 	memcpy(entry->spa, sta->addr, ETH_ALEN);
+	pmksa_cache_from_eapol_data(entry, sta->eapol_sm);
 
 	/* Replace an old entry for the same STA (if found) with the new entry
 	 */
@@ -959,7 +1037,7 @@ static void pmksa_cache_free(struct hostapd_data *hapd)
 	while (entry) {
 		prev = entry;
 		entry = entry->next;
-		free(prev);
+		_pmksa_cache_free_entry(prev);
 	}
 	eloop_cancel_timeout(pmksa_cache_expire, hapd, NULL);
 	for (i = 0; i < PMKID_HASH_SIZE; i++)
@@ -1372,6 +1450,7 @@ void wpa_free_station(struct sta_info *sta)
 
 	eloop_cancel_timeout(wpa_send_eapol_timeout, sm->hapd, sta);
 	eloop_cancel_timeout(wpa_sm_call_step, sm->hapd, sta->wpa_sm);
+	eloop_cancel_timeout(rsn_preauth_finished_cb, sm->hapd, sta);
 	free(sm->last_rx_eapol_key);
 	free(sm);
 	sta->wpa_sm = NULL;
@@ -2088,6 +2167,7 @@ SM_STATE(WPA_PTK, INITPMK)
 	if (sm->sta->pmksa) {
 		wpa_printf(MSG_DEBUG, "WPA: PMK from PMKSA cache");
 		memcpy(sm->PMK, sm->sta->pmksa->pmk, WPA_PMK_LEN);
+		pmksa_cache_to_eapol_data(sm->sta->pmksa, sm->sta->eapol_sm);
 	} else if ((key = ieee802_1x_get_key_crypt(sm->sta->eapol_sm, &len))) {
 		wpa_printf(MSG_DEBUG, "WPA: PMK from EAPOL state machine "
 			   "(len=%lu)", (unsigned long) len);
@@ -2294,6 +2374,8 @@ SM_STATE(WPA_PTK, PTKINITDONE)
 		       HOSTAPD_LEVEL_INFO, "pairwise key handshake completed "
 		       "(%s)",
 		       sm->sta->wpa == WPA_VERSION_WPA ? "WPA" : "RSN");
+	if (sm->sta->wpa_key_mgmt == WPA_KEY_MGMT_PSK)
+		accounting_sta_start(sm->hapd, sm->sta);
 }
 
 
